#!/usr/bin/env python
"""
This generates a Scatter plot Matrix for the given variables, the upper right
triangle of the matrix contains the correlation coefficient of the applicable pair
of variables

Example of use:
1) You can run the py file from a Python IDE or

2) save the py file in your "C:\Python24\Lib\site-packages\" folder
then call it from a program block as follows

BEGIN PROGRAM.
import compare2files as c2f
c2f.compare2files(file1 = "c:/Program files/SPSS/employee data.sav",
               file2 = "C:/temp/eee data2.sav",
               key = ['id'])
END PROGRAM.
"""
import spss, spssaux, spssdata, random

def _scatterMatrix(varlist, corr, title):
    nbVars = len(varlist)
    varnames = ' '.join(varlist)
    boxsize = 90.0 / nbVars
    cx = 5 # graph uses space from 5% to 95%
    cxlist = [cx]
    for i in range(nbVars-1):
        cxlist.append(cx + (i+1)*boxsize)
    cmd=r"""
    DO IF $CASENUM=1.
      COMPUTE dum01=0.
      COMPUTE dum02=0.
    END IF.

    * Chart Builder.
    GGRAPH
      /GRAPHDATASET NAME="graphdataset" VARIABLES=%(varnames)s dum01 dum02
      MISSING=VARIABLEWISE REPORTMISSING=NO
      /GRAPHSPEC SOURCE=INLINE LABEL="%(title)s".
    BEGIN GPL
    SOURCE: s=userSource(id("graphdataset"))
     DATA: dum01=col(source(s), name("dum01"))
     DATA: dum02=col(source(s), name("dum02"))
    """ % locals()

    for x in range(nbVars):
        cmd = cmd + ' DATA: %s=col(source(s), name("%s"))\n' % (varlist[x],varlist[x])
    cnt = 0
    for (x,y) in [(x,y) for x in range(nbVars) for y in range(nbVars)]:
        labpoint = ''
        varx = varlist[x]
        vary = varlist[y]
        if x > y:
            varx = 'dum01'
            vary = 'dum02'
            labpoint = ',label("%.3f"),color.exterior(color.white)' % corr[cnt]
            cnt += 1
        labx = ''
        laby = ''
        if x == 0:
            laby = ',label("%s")' % vary
        if y == nbVars-1:
            labx = ',label("%s")' % varx
        cx = cxlist[x]
        cy = cxlist[y]

        cmd = cmd  + """
        COMMENT: graph %(x)s,%(y)s
        GRAPH: begin(origin(%(cx)s%%,%(cy)s%%),scale(%(boxsize)s%%,%(boxsize)s%%))
        GUIDE: axis(dim(1),ticks(null()) %(labx)s)
        GUIDE: axis(dim(2),ticks(null()) %(laby)s)
        ELEMENT: point(position(%(varx)s*%(vary)s) %(labpoint)s)
        GRAPH: end() \n""" % locals()
    cmd = cmd + ' END GPL.\n'

    try:
        spss.Submit(cmd)
    except:
        print '\n *** Following syntax generated an error:\n',cmd
        raise

def _getCorrelations(vnames=None):
    # protect active dataset
    dsn = spssaux.GetActiveDatasetName()
    if not dsn:
        dsn = "S" + str(random.uniform(0,1))
        spss.Submit("dataset name %s window=asis." % dsn)

    cmd = r"""
    CORRELATIONS
      /VARIABLES=%(vnames)s
      /PRINT=TWOTAIL SIG
      /MISSING=PAIRWISE .""" %locals()
    datasetName, err = spssaux.CreateDatasetOutput(
                               cmd,
                               omsid='Correlations',
                               subtype='Correlations')
    spss.Submit("""
        DATASET ACTIVATE %s.
        SELECT IF Var2='Pearson Correlation'.
        """ % datasetName )
    data = spssdata.Spssdata()
    corrLab = []
    nbVars = len(vnames.split())
    i = 0
    for row in data:
        for idx in range(6+i,nbVars + 5):
            print idx, row[idx]
            corrLab.append(row[idx])
        i += 1
    data.close()
    spss.Submit('DATASET ACTIVATE %s.' % dsn)
    return corrLab

def doScatter(vnames, title=''):
    """Creates a Scatter Matrix for variables contained in vnames
    The upper right triangle of matrix contains correlation coefficients
    of applicable pair of variables
    """
    _corrLab = getCorrelations(vnames)
    _scatterMatrix(vnames.split(), corrLab, title)

if __name__ == '__main__':
    spss.Submit(r"GET FILE='c:/Program Files/SPSS/employee data.sav'.")
    vnames = 'salary salbegin prevexp educ'
    doScatter(vnames.split(), title='Scatter matrix with correlation coef.')
